#! /bin/bash
shopt -s extglob
THIS_DIR=$(dirname $(readlink -f $0))

. $THIS_DIR/setup_utils.sh

KOMPIRAD_LOG=$KOMPIRA_LOG_DIR/kompirad.log
#
# Kompiraのシステムインポートデータは 'en' と 'ja' の言語のみサポートしている。
# LANG が ja* 以外の場合は、 'en' をインストールする
#
LANGUAGE_CODE='en'
case "$LANG" in
    ja*) LANGUAGE_CODE="ja" ;;
    *) ;;
esac

check_service_status()
{
    #
    # kompirad が起動中は処理させない
    #
    if is_active_service kompirad; then
        echo_error "kompirad is running!"
        exit 1
    fi
    #
    # pgsql の recovery mode が 'f' のときだけ DB を操作できる
    #
    timeout=20
    while [ $timeout -gt 0 ]; do
        let timeout=$timeout-1 1
        if [ "$(pgsql_is_in_recovery)" == "f" ]; then
            break
        fi
        echo_warn "Waiting for PostgreSQL become a master..."
        sleep 1
    done
    if [ $timeout -eq 0 ]; then
        echo_error "PostgreSQL is recovery mode!"
        exit 1
    fi
    echo_info "PostgreSQL is running"
}

check_kompira_database()
{
    #
    # kompira DB にテーブルがなければ初期化する
    #
    if ! pgsql_has_kompira_table; then
        touch $KOMPIRA_VAR_DIR/startup/{do_migrate,do_import_data,do_update_packages_info}
    fi
}

do_migrate()
{
    #
    # $KOMPIRA_VAR_DIR/startup/do_migrate があれば、データベースを更新する
    #
    if [ -f $KOMPIRA_VAR_DIR/startup/do_migrate ]; then
        echo_info "Install pgcrypto extension."
        psql -U $PGSQL_USER -d $KOMPIRA_PG_DATABASE -c "CREATE EXTENSION IF NOT EXISTS pgcrypto"
        echo_info "Migrate database."
        $KOMPIRA_BIN/manage.py migrate -v1 --noinput
        if [ -f $KOMPIRA_VAR_DIR/startup/do_restore ]; then
            if [ -f $KOMPIRA_BACKUP ]; then
                echo_info "Restore backup data: $KOMPIRA_BACKUP"
                if ! $KOMPIRA_BIN/manage.py loaddata --traceback --ignorenonexistent $KOMPIRA_BACKUP; then
                    echo_error "Failed to restore backup"
                fi
            else
                echo_warn "Backup not found: $KOMPIRA_BACKUP"
            fi
            rm -f $KOMPIRA_VAR_DIR/startup/do_restore
        fi
        if [ "$($KOMPIRA_BIN/manage.py dumpdata --pks 1 kompira.user)" == "[]" ]; then
            echo_info "Load initial fixtures."
            $KOMPIRA_BIN/manage.py loaddata -v1 bootstrap_data-$LANGUAGE_CODE.json
        fi
        rm -f $KOMPIRA_VAR_DIR/startup/do_migrate
    fi
}

do_import_data()
{
    #
    # $KOMPIRA_VAR_DIR/startup/do_import_data があれば、ファイルをインポートする
    #
    if [ -f $KOMPIRA_VAR_DIR/startup/do_import_data ]; then
        echo_info "Import essential objects."
        $KOMPIRA_BIN/manage.py import_data --overwrite --force-mode $KOMPIRA_VAR_DIR/exported/{types-$LANGUAGE_CODE.dat,startup-$LANGUAGE_CODE.dat,nodetypes.dat}
        rm -f $KOMPIRA_VAR_DIR/startup/do_import_data
    fi
}

do_update_packages_info()
{
    #
    # $KOMPIRA_VAR_DIR/startup/do_update_packages_info があれば、パッケージ情報を更新する
    #
    if [ -f $KOMPIRA_VAR_DIR/startup/do_update_packages_info ]; then
        echo_info "Collect and update packages information."
        $KOMPIRA_BIN/manage.py packages_info --collect --update --traceback && rm -f $KOMPIRA_VAR_DIR/startup/do_update_packages_info || true
    fi
}

set -eu
set -o pipefail
exec >>$KOMPIRAD_LOG 2>&1
echo_info "starting pre-start of kompirad [$LANGUAGE_CODE]"
check_service_status
check_kompira_database
do_migrate
do_import_data
do_update_packages_info
